@@ -1,13 +1,27 @@
 from __future__ import print_function
 import unittest
+try:
+    unittest.skip
+except AttributeError:
+    import unittest2 as unittest
 import doctest
 import os
 import commands
 import cPickle
 from StringIO import StringIO
+import subprocess
+import sys
+
+try:
+    import pysam
+except ImportError:
+    pysam = None
 
 import vcf
-from vcf import utils
+from vcf import model, utils
+
+IS_PYTHON2 = sys.version_info[0] == 2
+IS_NOT_PYPY = 'PyPy' not in sys.version
 
 suite = doctest.DocTestSuite(vcf)
 
@@ -100,6 +114,37 @@ class TestVcfSpecs(unittest.TestCase):
                 print(c)
                 assert c
 
+    def test_vcf_4_2(self):
+        reader = vcf.Reader(fh('example-4.2.vcf'))
+        self.assertEqual(reader.metadata['fileformat'],  'VCFv4.2')
+
+        # If INFO contains no Source and Version keys, they should be None.
+        self.assertEqual(reader.infos['DP'].source, None)
+        self.assertEqual(reader.infos['DP'].version, None)
+
+        # According to spec, INFO Version key is required to be double quoted,
+        # but at least SAMtools 1.0 does not quote it. So we want to be
+        # forgiving here.
+        self.assertEqual(reader.infos['VDB'].source, None)
+        self.assertEqual(reader.infos['VDB'].version, '3')
+
+        # test we can walk the file at least
+        for r in reader:
+            for c in r:
+                assert c
+
+    def test_contig_idonly(self):
+        """Test VCF inputs with ##contig inputs containing only IDs. produced by bcftools 1.2+
+        """
+        reader = vcf.Reader(fh("contig_idonly.vcf"))
+        for cid, contig in reader.contigs.items():
+            if cid == "1":
+                assert contig.length is None
+            elif cid == "2":
+                assert contig.length == 2000
+            elif cid == "3":
+                assert contig.length == 3000
+
 class TestGatkOutput(unittest.TestCase):
 
     filename = 'gatk.vcf'
@@ -184,6 +229,34 @@ class TestBcfToolsOutput(unittest.TestCase):
             for s in r.samples:
                 s.phased
 
+class TestIssue214(unittest.TestCase):
+    """ See https://github.com/jamescasbon/PyVCF/issues/214 """
+
+    def test_issue_214_is_snp(self):
+        reader=vcf.Reader(fh('issue-214.vcf'))
+        r=next(reader)
+        self.assertTrue(r.is_snp)
+
+    def test_issue_214_var_type(self):
+        reader=vcf.Reader(fh('issue-214.vcf'))
+        r=next(reader)
+        self.assertEqual(r.var_type,'snp')
+
+    # Can the ref even be a spanning deletion?
+    # Note, this does not trigger issue 214, but I've added it here for completeness
+    def test_issue_214_ref_is_del_is_snp(self):
+        reader=vcf.Reader(fh('issue-214.vcf'))
+        next(reader)
+        r=next(reader)
+        self.assertTrue(r.is_snp)
+
+    # Can the ref even be a spanning deletion?
+    # Note, this does not trigger issue 214, but I've added it here for completeness
+    def test_issue_214_ref_is_del_var_type(self):
+        reader=vcf.Reader(fh('issue-214.vcf'))
+        next(reader)
+        r=next(reader)
+        self.assertEqual(r.var_type,'snp')
 
 class Test1kg(unittest.TestCase):
 
@@ -248,6 +321,16 @@ class TestGoNL(unittest.TestCase):
         self.assertEqual(reader.contigs['1'].length, 249250621)
 
 
+class TestStringAsFlag(unittest.TestCase):
+
+    def test_string_as_flag(self):
+        """A flag INFO field is declared as string (not allowed by the spec,
+        but seen in practice)."""
+        reader = vcf.Reader(fh('string_as_flag.vcf', 'r'))
+        for _ in reader:
+            pass
+
+
 class TestInfoOrder(unittest.TestCase):
 
     def _assert_order(self, definitions, fields):
@@ -310,6 +393,38 @@ class TestInfoTypeCharacter(unittest.TestCase):
             self.assertEquals(l.INFO, r.INFO)
 
 
+class TestParseMetaLine(unittest.TestCase):
+    def test_parse(self):
+        reader = vcf.Reader(fh('parse-meta-line.vcf'))
+        f = reader.metadata['MYFIELD'][0]
+        self.assertEqual(f['ID'], 'SomeField')
+        self.assertEqual(f['Version'], '3.4-0-g7e26428')
+        self.assertEqual(f['Date'], '"Wed Oct 07 09:11:47 CEST 2015"')
+        self.assertEqual(f['Options'], '"< 4 and > 3"')
+        next(reader)
+
+    def test_write(self):
+        reader = vcf.Reader(fh('parse-meta-line.vcf'))
+        out = StringIO()
+        writer = vcf.Writer(out, reader)
+
+        records = list(reader)
+
+        for record in records:
+            writer.write_record(record)
+        out.seek(0)
+        reader2 = vcf.Reader(out)
+
+        f = reader2.metadata['MYFIELD'][0]
+        self.assertEqual(f['ID'], 'SomeField')
+        self.assertEqual(f['Version'], '3.4-0-g7e26428')
+        self.assertEqual(f['Date'], '"Wed Oct 07 09:11:47 CEST 2015"')
+        self.assertEqual(f['Options'], '"< 4 and > 3"')
+
+        for l, r in zip(records, reader2):
+            self.assertEquals(l.INFO, r.INFO)
+
+
 class TestGatkOutputWriter(unittest.TestCase):
 
     def testWrite(self):
@@ -402,6 +517,27 @@ class TestSamplesSpace(unittest.TestCase):
         self.assertEqual(self.reader.samples, self.samples)
 
 
+class TestMetadataWhitespace(unittest.TestCase):
+    filename = 'metadata-whitespace.vcf'
+    def test_metadata_whitespace(self):
+        """
+        Test parsing metadata header lines with whitespace.
+        """
+        self.reader = vcf.Reader(fh(self.filename))
+
+        # Pick one INFO line and assert that we parsed it correctly.
+        info_indel = self.reader.infos['INDEL']
+        assert info_indel.id == 'INDEL'
+        assert info_indel.num == 0
+        assert info_indel.type == 'Flag'
+        assert info_indel.desc == 'Indicates that the variant is an INDEL.'
+
+        # Test we can walk the file at least.
+        for r in self.reader:
+            for c in r:
+                pass
+
+
 class TestMixedFiltering(unittest.TestCase):
     filename = 'mixed-filtering.vcf'
     def test_mixed_filtering(self):
@@ -426,7 +562,7 @@ class TestRecord(unittest.TestCase):
             self.assertEqual(len(var.samples), num_calls)
 
     def test_dunder_eq(self):
-        rec = vcf.Reader(fh('example-4.0.vcf')).next()
+        rec = next(vcf.Reader(fh('example-4.0.vcf')))
         self.assertFalse(rec == None)
         self.assertFalse(None == rec)
 
@@ -459,6 +595,13 @@ class TestRecord(unittest.TestCase):
                 self.assertEqual([0.0/6.0], aaf)
             elif var.POS == 1234567:
                 self.assertEqual([2.0/4.0, 1.0/4.0], aaf)
+        reader = vcf.Reader(fh('example-4.1-ploidy.vcf'))
+        for var in reader:
+            aaf = var.aaf
+            if var.POS == 60034:
+                self.assertEqual([4.0/6.0], aaf)
+            elif var.POS == 60387:
+                self.assertEqual([1.0/3.0], aaf)
 
     def test_pi(self):
         reader = vcf.Reader(fh('example-4.0.vcf'))
@@ -510,6 +653,24 @@ class TestRecord(unittest.TestCase):
             elif var.POS == 1234567:
                 self.assertEqual(False, is_snp)
 
+
+    def test_is_snp_for_n_alt(self):
+        record = model._Record(
+                '1',
+                10,
+                'id1',
+                'C',
+                [model._Substitution('N')],
+                None,
+                None,
+                {},
+                None,
+                {},
+                None
+        )
+        self.assertTrue(record.is_snp)
+
+
     def test_is_indel(self):
         reader = vcf.Reader(fh('example-4.0.vcf'))
         for var in reader:
@@ -731,7 +892,7 @@ class TestRecord(unittest.TestCase):
 
     def test_info_multiple_values(self):
         reader = vcf.Reader(fh('example-4.1-info-multiple-values.vcf'))
-        var = reader.next()
+        var = next(reader)
         # check Float type INFO field with multiple values
         expected = [19.3, 47.4, 14.0]
         actual = var.INFO['RepeatCopies']
@@ -751,11 +912,244 @@ class TestRecord(unittest.TestCase):
             self.assertEqual(cPickle.loads(cPickle.dumps(var)), var)
 
 
+    def assert_has_expected_coordinates(
+            self,
+            record,
+            expected_coordinates,
+            expected_affected_coordinates
+        ):
+        self.assertEqual(
+                (record.start, record.end),
+                expected_coordinates
+        )
+        self.assertEqual(
+                (record.affected_start, record.affected_end),
+                expected_affected_coordinates
+        )
+
+
+    def test_coordinates_for_snp(self):
+        record = model._Record(
+                '1',
+                10,
+                'id1',
+                'C',
+                [model._Substitution('A')],
+                None,
+                None,
+                {},
+                None,
+                {},
+                None
+        )
+        self.assert_has_expected_coordinates(record, (9, 10), (9, 10))
+
+
+    def test_coordinates_for_insertion(self):
+        record = model._Record(
+                '1',
+                10,
+                'id2',
+                'C',
+                [model._Substitution('CTA')],
+                None,
+                None,
+                {},
+                None,
+                {},
+                None
+        )
+        self.assert_has_expected_coordinates(record, (9, 10), (10, 10))
+
+
+    def test_coordinates_for_deletion(self):
+        record = model._Record(
+                '1',
+                10,
+                'id3',
+                'CTA',
+                [model._Substitution('C')],
+                None,
+                None,
+                {},
+                None,
+                {},
+                None
+        )
+        self.assert_has_expected_coordinates(record, (9, 12), (10, 12))
+
+
+    def test_coordinates_for_None_alt(self):
+        record = model._Record(
+                '1',
+                10,
+                'id4',
+                'C',
+                [None],
+                None,
+                None,
+                {},
+                None,
+                {},
+                None
+        )
+        self.assert_has_expected_coordinates(record, (9, 10), (9, 10))
+
+
+    def test_coordinates_for_multiple_snps(self):
+        record = model._Record(
+                '1',
+                10,
+                'id5',
+                'C',
+                [
+                    model._Substitution('A'),
+                    model._Substitution('G'),
+                    model._Substitution('T')
+                ],
+                None,
+                None,
+                {},
+                None,
+                {},
+                None
+        )
+        self.assert_has_expected_coordinates(record, (9, 10), (9, 10))
+
+
+    def test_coordinates_for_insert_and_snp(self):
+        record = model._Record(
+                '1',
+                10,
+                'id6',
+                'C',
+                [
+                    model._Substitution('GTA'),
+                    model._Substitution('G'),
+                ],
+                None,
+                None,
+                {},
+                None,
+                {},
+                None
+        )
+        self.assert_has_expected_coordinates(record, (9, 10), (9, 10))
+        record = model._Record(
+                '1',
+                10,
+                'id7',
+                'C',
+                [
+                    model._Substitution('G'),
+                    model._Substitution('GTA'),
+                ],
+                None,
+                None,
+                {},
+                None,
+                {},
+                None
+        )
+        self.assert_has_expected_coordinates(record, (9, 10), (9, 10))
+
+
+    def test_coordinates_for_snp_and_deletion(self):
+        record = model._Record(
+                '1',
+                10,
+                'id8',
+                'CTA',
+                [
+                    model._Substitution('C'),
+                    model._Substitution('CTG'),
+                ],
+                None,
+                None,
+                {},
+                None,
+                {},
+                None
+        )
+        self.assert_has_expected_coordinates(record, (9, 12), (10, 12))
+        record = model._Record(
+                '1',
+                10,
+                'id9',
+                'CTA',
+                [
+                    model._Substitution('CTG'),
+                    model._Substitution('C'),
+                ],
+                None,
+                None,
+                {},
+                None,
+                {},
+                None
+        )
+        self.assert_has_expected_coordinates(record, (9, 12), (10, 12))
+
+
+    def test_coordinates_for_insertion_and_deletion(self):
+        record = model._Record(
+                '1',
+                10,
+                'id10',
+                'CT',
+                [
+                    model._Substitution('CA'),
+                    model._Substitution('CTT'),
+                ],
+                None,
+                None,
+                {},
+                None,
+                {},
+                None
+        )
+        self.assert_has_expected_coordinates(record, (9, 11), (10, 11))
+        record = model._Record(
+                '1',
+                10,
+                'id11',
+                'CT',
+                [
+                    model._Substitution('CTT'),
+                    model._Substitution('CA'),
+                ],
+                None,
+                None,
+                {},
+                None,
+                {},
+                None
+        )
+        self.assert_has_expected_coordinates(record, (9, 11), (10, 11))
+
+
+    def test_coordinates_for_breakend(self):
+        record = model._Record(
+                '1',
+                10,
+                'id12',
+                'CTA',
+                [model._Breakend('1', 500, False, True, 'GGTC', True)],
+                None,
+                None,
+                {},
+                None,
+                {},
+                None
+        )
+        self.assert_has_expected_coordinates(record, (9, 12), (9, 12))
+
+
 class TestCall(unittest.TestCase):
 
     def test_dunder_eq(self):
         reader = vcf.Reader(fh('example-4.0.vcf'))
-        var = reader.next()
+        var = next(reader)
         example_call = var.samples[0]
         self.assertFalse(example_call == None)
         self.assertFalse(None == example_call)
@@ -808,40 +1202,73 @@ class TestCall(unittest.TestCase):
                 self.assertEqual([None,1,2], gt_types)
 
 
-class TestTabix(unittest.TestCase):
+@unittest.skipUnless(pysam, "test requires installation of PySAM.")
+class TestFetch(unittest.TestCase):
 
     def setUp(self):
         self.reader = vcf.Reader(fh('tb.vcf.gz', 'rb'))
 
-        self.run = vcf.parser.pysam is not None
+
+    def assertFetchedExpectedPositions(
+            self, fetched_variants, expected_positions):
+        fetched_positions = [var.POS for var in fetched_variants]
+        self.assertEqual(fetched_positions, expected_positions)
+
+
+    def testNoVariantsInRange(self):
+        fetched_variants = self.reader.fetch('20', 14370, 17329)
+        self.assertFetchedExpectedPositions(fetched_variants, [])
+
+
+    def testNoVariantsForZeroLengthInterval(self):
+        fetched_variants = self.reader.fetch('20', 14369, 14369)
+        self.assertFetchedExpectedPositions(fetched_variants, [])
 
 
     def testFetchRange(self):
-        if not self.run:
-            return
-        lines = list(self.reader.fetch('20', 14370, 14370))
-        self.assertEquals(len(lines), 1)
-        self.assertEqual(lines[0].POS, 14370)
+        fetched_variants = self.reader.fetch('20', 14369, 14370)
+        self.assertFetchedExpectedPositions(fetched_variants, [14370])
 
-        lines = list(self.reader.fetch('20', 14370, 17330))
-        self.assertEquals(len(lines), 2)
-        self.assertEqual(lines[0].POS, 14370)
-        self.assertEqual(lines[1].POS, 17330)
+        fetched_variants = self.reader.fetch('20', 14369, 17330)
+        self.assertFetchedExpectedPositions(
+                fetched_variants, [14370, 17330])
+
+        fetched_variants = self.reader.fetch('20', 1110695, 1234567)
+        self.assertFetchedExpectedPositions(
+                fetched_variants, [1110696, 1230237, 1234567])
 
 
-        lines = list(self.reader.fetch('20', 1110695, 1234567))
-        self.assertEquals(len(lines), 3)
-
-    def testFetchSite(self):
-        if not self.run:
-            return
-        site = self.reader.fetch('20', 14370)
-        self.assertEqual(site.POS, 14370)
-
-        site = self.reader.fetch('20', 14369)
-        assert site is None
+    def testFetchesFromStartIfStartOnlySpecified(self):
+        fetched_variants = self.reader.fetch('20', 1110695)
+        self.assertFetchedExpectedPositions(
+                fetched_variants, [1110696, 1230237, 1234567])
 
 
+    def testFetchesAllFromChromIfOnlyChromSpecified(self):
+        fetched_variants = self.reader.fetch('20')
+        self.assertFetchedExpectedPositions(
+                fetched_variants,
+                [14370, 17330, 1110696, 1230237, 1234567]
+        )
+
+
+@unittest.skipUnless(pysam, "test requires installation of PySAM.")
+class TestIssue201(unittest.TestCase):
+    def setUp(self):
+        # This file contains some non-ASCII characters in a UTF-8 encoding.
+        # https://github.com/jamescasbon/PyVCF/issues/201
+        self.reader = vcf.Reader(fh('issue-201.vcf.gz', 'rb'),
+                                 encoding='utf-8')
+
+    def testIterate(self):
+        for record in self.reader:
+            # Should not raise decoding errors.
+            pass
+
+    def testFetch(self):
+        for record in self.reader.fetch(chrom='17'):
+            # Should not raise decoding errors.
+            pass
 
 
 class TestOpenMethods(unittest.TestCase):
@@ -870,12 +1297,61 @@ class TestOpenMethods(unittest.TestCase):
         self.assertEqual(self.samples, r.samples)
 
 
+class TestSampleFilter(unittest.TestCase):
+    @unittest.skipUnless(IS_PYTHON2, "test broken for Python 3")
+    def testCLIListSamples(self):
+        proc = subprocess.Popen('python scripts/vcf_sample_filter.py vcf/test/example-4.1.vcf', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        out, err = proc.communicate()
+        self.assertEqual(proc.returncode, 0)
+        self.assertFalse(err)
+        expected_out = ['Samples:', '0: NA00001', '1: NA00002', '2: NA00003']
+        self.assertEqual(out.splitlines(), expected_out)
+
+    @unittest.skipUnless(IS_PYTHON2, "test broken for Python 3")
+    def testCLIWithFilter(self):
+        proc = subprocess.Popen('python scripts/vcf_sample_filter.py vcf/test/example-4.1.vcf -f 1,2 --quiet', shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        out, err = proc.communicate()
+        self.assertEqual(proc.returncode, 0)
+        self.assertTrue(out)
+        self.assertFalse(err)
+        buf = StringIO()
+        buf.write(out)
+        buf.seek(0)
+        #print(buf.getvalue())
+        reader = vcf.Reader(buf)
+        self.assertEqual(reader.samples, ['NA00001'])
+        rec = next(reader)
+        self.assertEqual(len(rec.samples), 1)
+
+    @unittest.skipUnless(IS_NOT_PYPY, "test broken for PyPy")
+    def testSampleFilterModule(self):
+        # init filter with filename, get list of samples
+        filt = vcf.SampleFilter('vcf/test/example-4.1.vcf')
+        self.assertEqual(filt.samples, ['NA00001', 'NA00002', 'NA00003'])
+        # set filter, check which samples will be kept
+        filtered = filt.set_filters(filters="0", invert=True)
+        self.assertEqual(filtered, ['NA00001'])
+        # write filtered file to StringIO
+        buf = StringIO()
+        filt.write(buf)
+        buf.seek(0)
+        #print(buf.getvalue())
+        # undo monkey patch by destroying instance
+        del filt
+        self.assertTrue('sample_filter' not in dir(vcf.Reader))
+        # read output
+        reader = vcf.Reader(buf)
+        self.assertEqual(reader.samples, ['NA00001'])
+        rec = next(reader)
+        self.assertEqual(len(rec.samples), 1)
+
+
 class TestFilter(unittest.TestCase):
 
 
+    @unittest.skip("test currently broken")
     def testApplyFilter(self):
         # FIXME: broken with distribute
-        return
         s, out = commands.getstatusoutput('python scripts/vcf_filter.py --site-quality 30 test/example-4.0.vcf sq')
         #print(out)
         self.assertEqual(s, 0)
@@ -903,9 +1379,9 @@ class TestFilter(unittest.TestCase):
         self.assertEqual(n, 2)
 
 
+    @unittest.skip("test currently broken")
     def testApplyMultipleFilters(self):
         # FIXME: broken with distribute
-        return
         s, out = commands.getstatusoutput('python scripts/vcf_filter.py --site-quality 30 '
         '--genotype-quality 50 test/example-4.0.vcf sq mgq')
         self.assertEqual(s, 0)
@@ -925,7 +1401,7 @@ class TestRegression(unittest.TestCase):
 
     def test_issue_16(self):
         reader = vcf.Reader(fh('issue-16.vcf'))
-        n = reader.next()
+        n = next(reader)
         assert n.QUAL == None
 
     def test_null_mono(self):
@@ -940,7 +1416,7 @@ class TestRegression(unittest.TestCase):
         out.seek(0)
         print(out.getvalue())
         p2 = vcf.Reader(out)
-        rec = p2.next()
+        rec = next(p2)
         assert rec.samples
 
 
@@ -1014,26 +1490,105 @@ class TestGATKMeta(unittest.TestCase):
         self.assertEqual(reader.metadata['GATKCommandLine'][1]['CommandLineOptions'], '"analysis_type=VariantAnnotator annotation=[HomopolymerRun, VariantType, TandemRepeatAnnotator]"')
 
 
+
+class TestUncalledGenotypes(unittest.TestCase):
+    """Test the handling of uncalled (., ./.) sample genotypes."""
+
+    def test_read_uncalled(self):
+        """Test that uncalled genotypes are properly read into
+        gt_nums, gt_bases, ploidity, and gt_alleles properties
+        of _Call objects.  For uncalled _Call objects:
+
+        - gt_nums should be None
+        - gt_bases should be None
+        - ploidity should match the input ploidity
+        - gt_alleles should be a list of None's with length
+          matching the ploidity"""
+
+        reader = vcf.Reader(fh('uncalled_genotypes.vcf'))
+        for var in reader:
+            gt_bases = [s.gt_bases for s in var.samples]
+            gt_nums = [s.gt_nums for s in var.samples]
+            ploidity = [s.ploidity for s in var.samples]
+            gt_alleles = [s.gt_alleles for s in var.samples]
+
+            if var.POS == 14370:
+                self.assertEqual(['0|0', None, '1/1'], gt_nums)
+                self.assertEqual(['G|G', None, 'A/A'], gt_bases)
+                self.assertEqual([2,2,2], ploidity)
+                self.assertEqual([['0','0'], [None,None], ['1','1']], gt_alleles)
+            elif var.POS == 17330:
+                self.assertEqual([None, '0|1', '0/0'], gt_nums)
+                self.assertEqual([None, 'T|A', 'T/T'], gt_bases)
+                self.assertEqual([3,2,2], ploidity)
+                self.assertEqual([[None,None,None], ['0','1'], ['0','0']], gt_alleles)
+            elif var.POS == 1234567:
+                self.assertEqual(['0/1', '0/2', None], gt_nums)
+                self.assertEqual(['GTC/G', 'GTC/GTCT', None], gt_bases)
+                self.assertEqual([2,2,1], ploidity)
+                self.assertEqual([['0','1'], ['0','2'], [None]], gt_alleles)
+        reader._reader.close()
+
+
+    def test_write_uncalled(self):
+        """Test that uncalled genotypes are written just as
+        they were read in the input file."""
+
+        reader = vcf.Reader(fh('uncalled_genotypes.vcf'))
+
+        # Write all reader records to a stream.
+        out = StringIO()
+        writer = vcf.Writer(out, reader, lineterminator='\n')
+        for record in reader:
+            writer.write_record(record)
+        reader._reader.close()
+
+
+        # Compare the written stream to the input reader line-by-line.
+        out.seek(0)
+        out_lines = out.getvalue().split('\n')
+        in_file = fh('uncalled_genotypes.vcf')
+        in_lines = [l.rstrip('\n') for l in in_file]
+        in_file.close()
+        for (in_line, out_line) in zip(in_lines, out_lines):
+            self.assertEqual(in_line,out_line)
+
+class TestStrelka(unittest.TestCase):
+
+    def test_strelka(self):
+        reader = vcf.Reader(fh('strelka.vcf'))
+        n = next(reader)
+        assert n is not None
+
+
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestVcfSpecs))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestGatkOutput))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestFreebayesOutput))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestSamtoolsOutput))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestBcfToolsOutput))
+suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestIssue214))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(Test1kg))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(Test1kgSites))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestGoNL))
+suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestStringAsFlag))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestInfoOrder))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestInfoTypeCharacter))
+suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestParseMetaLine))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestGatkOutputWriter))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestBcfToolsOutputWriter))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestWriterDictionaryMeta))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestSamplesSpace))
+suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestMetadataWhitespace))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestMixedFiltering))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestRecord))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestCall))
-suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestTabix))
+suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestFetch))
+suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestIssue201))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestOpenMethods))
+suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestSampleFilter))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestFilter))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestRegression))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestUtils))
 suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestGATKMeta))
+suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestUncalledGenotypes))
+suite.addTests(unittest.TestLoader().loadTestsFromTestCase(TestStrelka))
